from azion.__consts__ import (
    DEFAULT_PARAMS, PUBLIC_ENDPOINT, USER_AGENT, VERSION)

from azion.models import (
    Domain, EdgeApplication, EdgeFunction, ErrorResponses, CacheSettings, Origin, Rule, Token,
    as_boolean, decode_json, filter_none, instance_from_data, many_of)

from http.client import HTTPConnection

import base64
import requests
import logging


class Client(requests.Session):

    def __init__(self, debug=False, **kwargs):
        requests.Session.__init__(self)
        self.base_url = PUBLIC_ENDPOINT
        self.session = requests.sessions.Session()
        self.headers = self.default_headers()

        self.debugIfEnabled(debug)

        username = kwargs.get('username')
        password = kwargs.get('password')
        token = kwargs.get('token')

        if token:
            self.use_token(token)
        elif username and password:
            token = self.generate_token(username, password)
            self.use_token(token)
        else:
            raise ValueError("Token or (username and password) must be provided")


    def default_headers(self):
        return {
            'Accept': 'application/json; version={}'.format(VERSION),
            'Accept-Charset': 'utf-8',
            'Content-Type': 'application/json',
            'User-Agent': '{}'.format(USER_AGENT)
        }


    def generate_token(self, username, password):
        raw = f'{username}:{password}'.encode('ascii')
        value = base64.b64encode(raw).decode('ascii')

        self.headers.update({
            'Authorization': 'Basic {}'.format(value)
        })

        url = self.build_url('tokens')
        response = self.post(url)
        json = decode_json(response, 201)

        return instance_from_data(Token, json).value


    def use_token(self, token):
        self.headers.update({
            'Authorization': 'Token {}'.format(token)
        })


    def build_url(self, *args, **kwargs):
        """Build a URL depending on the `base_url`
        attribute."""
        params = [kwargs.get('base_url') or self.base_url]
        params.extend(args)
        params = list(map(str, params))
        return '/'.join(params)


    def domains(self):
        payload = {'page_size': 100}
        url = self.build_url('domains')
        response = self.get(url, params = DEFAULT_PARAMS)
        json = decode_json(response, 200)

        return many_of(Domain, json['results'])


    def get_domain(self, domain_id):
        url = self.build_url('domains', domain_id)
        response = self.get(url)
        json = decode_json(response, 200)

        return instance_from_data(Domain, json['results'])


    def edge_applications(self):
        url = self.build_url('edge_applications')
        response = self.get(url, params = DEFAULT_PARAMS)
        json = decode_json(response, 200)

        return many_of(EdgeApplication, json['results'])


    def get_edge_application(self, edge_application_id):
        url = self.build_url('edge_applications', edge_application_id)
        response = self.get(url, params = DEFAULT_PARAMS)
        json = decode_json(response, 200)

        return instance_from_data(EdgeApplication, json['results'])


    def get_edge_application_origins(self, edge_application_id):
        url = self.build_url(
                'edge_applications',
                edge_application_id,
                'origins')

        response = self.get(url)
        json = decode_json(response, 200)

        return many_of(Origin, json['results'])

    def get_edge_application_origin(self, edge_application_id, origin_key):
        url = self.build_url(
                'edge_applications',
                edge_application_id,
                'origins',
                origin_key) # TODO: report api parameter adjust

        response = self.get(url)
        json = decode_json(response, 200)

        return instance_from_data(Origin, json['results'])

    def edge_functions(self):
        payload = {'page_size': 100}
        url = self.build_url('edge_functions')
        response = self.get(url, params = DEFAULT_PARAMS)
        json = decode_json(response, 200)

        return many_of(EdgeFunction, json['results'])


    def get_edge_function(self, edge_function_id):
        url = self.build_url('edge_functions', edge_function_id)
        response = self.get(url)
        json = decode_json(response, 200)

        return instance_from_data(EdgeFunction, json['results'])


    def debugIfEnabled(self, flag=False):
        self.debug = flag

        if flag:
            logging.basicConfig(level=logging.DEBUG)
            HTTPConnection.debuglevel = 1
        else:
            HTTPConnection.debuglevel = 0




import azion.api
import click
import os
import sys
from pprint import pprint

AZION_USERNAME = os.getenv('AZION_USERNAME')
AZION_PASSWORD = os.getenv('AZION_PASSWORD')
AZION_API_TOKEN = os.getenv('AZION_API_TOKEN')

if AZION_API_TOKEN:
    client = azion.api.Client(token=AZION_API_TOKEN)
elif AZION_USERNAME and AZION_PASSWORD:
    client = azion.api.Client(
        username=AZION_USERNAME,
        password=AZION_PASSWORD
    )
else:
    sys.tracebacklimit = 0
    raise ValueError("AZION_API_TOKEN or (AZION_USERNAME and AZION_PASSWORD) must be provided")

@click.group()
def edge_application_group():
    pass

@click.option('--debug', is_flag=True, help="Debug the api requests.")
@edge_application_group.command("edge-application-list")
def edge_application_list(debug):
    client.debugIfEnabled(debug)
    for edge_application in client.edge_applications():
        print(edge_application)

@click.option('--debug', is_flag=True, help="Debug the api requests.")
@edge_application_group.command("edge-application-info")
@click.option('--id', type=int, required=True)
def edge_application_info(debug, id):
    client.debugIfEnabled(debug)
    edge_application = client.get_edge_application(id)
    pprint(vars(edge_application))

##############################################################################

@click.group()
def origin_group():
    pass

@click.option('--debug', is_flag=True, help="Debug the api requests.")
@click.option('--edge-application-id', type=int, required=True)
@origin_group.command("origin-list")
def origin_list(debug, edge_application_id):
    client.debugIfEnabled(debug)
    for origin in client.get_edge_application_origins(edge_application_id):
        print(origin)

@click.option('--debug', is_flag=True, help="Debug the api requests.")
@origin_group.command("origin-info")
@click.option('--edge-application-id', type=int, required=True)
@click.option('--origin-key', required=True)
def origin_info(debug, edge_application_id, origin_key):
    client.debugIfEnabled(debug)
    origin = client.get_edge_application_origin(edge_application_id, origin_key)
    pprint(vars(origin))

##############################################################################

@click.group()
def domain_group():
    pass

@click.option('--debug', is_flag=True, help="Debug the api requests.")
@domain_group.command("domain-list")
def domain_list(debug):
    client.debugIfEnabled(debug)
    for domain in client.domains():
        print(domain)

@click.option('--debug', is_flag=True, help="Debug the api requests.")
@domain_group.command("domain-info")
@click.option('--id', type=int, required=True)
def domain_info(debug, id):
    client.debugIfEnabled(debug)
    domain = client.get_domain(id)
    pprint(vars(domain))

##############################################################################

@click.group()
def edge_function_group():
    pass

@click.option('--debug', is_flag=True, help="Debug the api requests.")
@edge_function_group.command("edge-function-list")
def edge_function_list(debug):
    client.debugIfEnabled(debug)
    for edge_function in client.edge_functions():
        print(edge_function)


@click.option('--debug', is_flag=True, help="Debug the api requests.")
@edge_function_group.command("edge-function-info")
@click.option('--id', type=int, required=True)
def edge_function_info(debug, id):
    client.debugIfEnabled(debug)
    edge_function = client.get_edge_function(id)
    pprint(vars(edge_function))

##############################################################################

command_sources = [
    domain_group,
    edge_function_group,
    edge_application_group,
    origin_group
]

cli = click.CommandCollection(sources=command_sources)

if __name__ == '__main__':
    cli()

